﻿using System;
using Stride.Core.Mathematics;

namespace ForceDirected
{
    public static class Forces
    {
        //constants
        public static AttractionForce Attraction;
        public static RepulsionForce Repulsion;

        public static Vector3 ConstantsEades;
        public static Vector2 ConstantsFR;
        public static Vector4 ConstantsSA;

        //public force functions
        public static Vector3 Attract(Vector3 diff)
        {
            switch (Attraction)
            {
                case AttractionForce.Eades: return AttractEades(diff);
                case AttractionForce.FruchtermanReingold: return AttractFR(diff);
                default: return AttractSpring(diff);
            }

        }

        public static Vector3 Repulse(Vector3 diff)
        {
            switch (Repulsion)
            {
                case RepulsionForce.Eades: return RepulseEades(diff);
                case RepulsionForce.FruchtermanReingold: return RepulseFR(diff);
                default: return RepulseAttractor(diff);
            }
        }

        //Eades
        private static Vector3 AttractEades(Vector3 diff)
        {
            var l = Math.Abs(Vector3.Dot(diff, diff));
            diff /= l;

            return diff * ConstantsEades.X * (float)Math.Log(l / ConstantsEades.Y);
        }

        private static Vector3 RepulseEades(Vector3 diff)
        {
            var l = Math.Abs(Vector3.Dot(diff, diff));
            diff /= l;
            return diff * Math.Min(ConstantsEades.Z / l, 10f);
        }

        //Fruchterman-Reingold
        private static Vector3 AttractFR(Vector3 diff)
        {
            var l = diff.Length();
            diff /= l;

            return diff * ((l * l) / ConstantsFR.X);
        }

        private static Vector3 RepulseFR(Vector3 diff)
        {
            var l = diff.Length();
            diff /= l;
            return diff * ((ConstantsFR.Y * ConstantsFR.Y) / l);
        }

        //SpringAttractor
        private static Vector3 AttractSpring(Vector3 diff)
        {
            var l = diff.Length();
            diff /= l;

            return diff * (l - ConstantsSA.X) * 0.5f;
        }

        private static Vector3 RepulseAttractor(Vector3 diff)
        {
            var l = diff.Length();

            if (l > ConstantsSA.Y) return new Vector3(0f);

            diff /= l;
            var s = (float)Math.Pow(l / ConstantsSA.Y, 1 / ConstantsSA.Z);
            var f = s * 9f * ConstantsSA.W * (1 / (s + 1) + ((s - 3) / 4)) / l;
            return diff * f;
        }
    }
}
